@@ -9,12 +9,14 @@ use Path::Tiny;
 use Try::Tiny;
 use Moo;
 use Module::CoreList;
+use Scalar::Util qw(blessed);
 
 use Carton;
 use Carton::Builder;
 use Carton::Mirror;
 use Carton::Lock;
 use Carton::Util;
+use Carton::Environment;
 use Carton::Error;
 use Carton::Requirements;
 
@@ -26,14 +28,16 @@ has verbose => (is => 'rw');
 has carton  => (is => 'lazy');
 has mirror  => (is => 'rw', builder => 1,
                 coerce => sub { Carton::Mirror->new($_[0]) });
+has environment => (is => 'lazy',
+                    handles => [ qw( cpanfile lockfile install_path vendor_cache )]);
 
 sub _build_mirror {
     my $self = shift;
     $ENV{PERL_CARTON_MIRROR} || $Carton::Mirror::DefaultMirror;
 }
 
-sub install_path {
-    Path::Tiny->new($ENV{PERL_CARTON_PATH} || 'local')->absolute;
+sub _build_environment {
+    Carton::Environment->build;
 }
 
 sub work_file {
@@ -43,10 +47,6 @@ sub work_file {
     $wf;
 }
 
-sub vendor_cache {
-    Path::Tiny->new("vendor/cache")->absolute;
-}
-
 sub run {
     my($self, @args) = @_;
 
@@ -64,16 +64,21 @@ sub run {
     push @commands, @args;
 
     my $cmd = shift @commands || 'install';
-    my $call = $self->can("cmd_$cmd");
 
     my $code = try {
-        $self->error("Could not find command '$cmd'\n")
-            unless $call;
+        my $call = $self->can("cmd_$cmd")
+          or Carton::Error::CommandNotFound->throw(error => "Could not find command '$cmd'");
         $self->$call(@commands);
         return 0;
     } catch {
-        ref =~ /Carton::Error::CommandExit/ and return 255;
-        die $_;
+        die $_ unless blessed $_ && $_->can('rethrow');
+
+        if ($_->isa('Carton::Error::CommandExit')) {
+            return $_->code || 255;
+        } elsif ($_->isa('Carton::Error')) {
+            warn $_->error, "\n";
+            return 255;
+        }
     };
 
     return $code;
@@ -152,19 +157,16 @@ sub cmd_version {
 sub cmd_bundle {
     my($self, @args) = @_;
 
-    my $lock = $self->find_lock;
-    my $cpanfile = $self->find_cpanfile;
+    my $lock = $self->lockfile->load;
+    my $cpanfile = $self->cpanfile;
 
-    if ($lock) {
-        $self->print("Bundling modules using $cpanfile\n");
+    $self->print("Bundling modules using $cpanfile\n");
 
-        my $builder = Carton::Builder->new(
-            mirror => $self->mirror,
-        );
-        $builder->bundle($self->install_path, $self->vendor_cache, $lock);
-    } else {
-        $self->error("Can't locate carton.lock file. Run carton install first\n");
-    }
+    my $builder = Carton::Builder->new(
+        mirror => $self->mirror,
+        cpanfile => $self->cpanfile,
+    );
+    $builder->bundle($self->install_path, $self->vendor_cache, $lock);
 
     $self->printf("Complete! Modules were bundled into %s\n", $self->vendor_cache, SUCCESS);
 }
@@ -183,18 +185,19 @@ sub cmd_install {
         "cached!"     => \my $cached,
     );
 
-    my $lock = $self->find_lock;
+    my $lock = $self->lockfile->load_if_exists;
 
     if ($deployment && !$lock) {
         $self->error("--deployment requires carton.lock: Run `carton install` and make sure carton.lock is checked into your version control.\n");
     }
 
-    my $cpanfile = $self->find_cpanfile;
+    my $cpanfile = $self->cpanfile;
 
     my $builder = Carton::Builder->new(
         cascade => 1,
         mirror  => $self->mirror,
         without => \@without,
+        cpanfile => $self->cpanfile,
     );
 
     # TODO: --without with no .lock won't fetch the groups, resulting in insufficient requirements
@@ -220,7 +223,7 @@ sub cmd_install {
 
     unless ($deployment) {
         my $prereqs = Module::CPANfile->load($cpanfile)->prereqs;
-        Carton::Lock->build_from_local($path, $prereqs)->write($self->lock_file);
+        Carton::Lock->build_from_local($path, $prereqs)->write($self->lockfile);
     }
 
     $self->print("Complete! Modules were installed into $path\n", SUCCESS);
@@ -229,8 +232,7 @@ sub cmd_install {
 sub cmd_show {
     my($self, @args) = @_;
 
-    my $lock = $self->find_lock
-        or $self->error("Can't find carton.lock: Run `carton install`\n");
+    my $lock = $self->lockfile->load;
 
     for my $module (@args) {
         my $dist = $lock->find($module)
@@ -249,8 +251,7 @@ sub cmd_list {
         "distfile" => sub { $format = 'distfile' },
     );
 
-    my $lock = $self->find_lock
-        or $self->error("Can't find carton.lock: Run `carton install` to rebuild the lock file.\n");
+    my $lock = $self->lockfile->load;
 
     for my $dist ($lock->distributions) {
         $self->print($dist->$format . "\n");
@@ -260,10 +261,9 @@ sub cmd_list {
 sub cmd_tree {
     my($self, @args) = @_;
 
-    my $lock = $self->find_lock
-      or $self->error("Can't find carton.lock: Run `carton install` to rebuild the lock file.\n");
+    my $lock = $self->lockfile->load;
 
-    my $cpanfile = Module::CPANfile->load($self->find_cpanfile);
+    my $cpanfile = Module::CPANfile->load($self->cpanfile);
     my $requirements = Carton::Requirements->new(lock => $lock, prereqs => $cpanfile->prereqs);
 
     my %seen;
@@ -279,10 +279,9 @@ sub cmd_tree {
 sub cmd_check {
     my($self, @args) = @_;
 
-    my $lock = $self->find_lock
-      or $self->error("Can't find carton.lock: Run `carton install` to rebuild the lock file.\n");
+    my $lock = $self->lockfile->load;
 
-    my $prereqs = Module::CPANfile->load($self->find_cpanfile)->prereqs;
+    my $prereqs = Module::CPANfile->load($self->cpanfile)->prereqs;
 
     # TODO remove $lock
     # TODO pass git spec to Requirements?
@@ -323,7 +322,7 @@ sub cmd_check {
 sub cmd_update {
     my($self, @args) = @_;
 
-    my $cpanfile = Module::CPANfile->load($self->find_cpanfile);
+    my $cpanfile = Module::CPANfile->load($self->cpanfile);
     my $prereqs = $cpanfile->prereqs;
 
     my $reqs = CPAN::Meta::Requirements->new;
@@ -332,8 +331,7 @@ sub cmd_update {
 
     @args = grep { $_ ne 'perl' } $reqs->required_modules unless @args;
 
-    my $lock = $self->find_lock
-        or $self->error("Can't find carton.lock: Run `carton install` to build the lock file.\n");
+    my $lock = $self->lockfile->load;
 
     my @modules;
     for my $module (@args) {
@@ -345,17 +343,17 @@ sub cmd_update {
 
     my $builder = Carton::Builder->new(
         mirror => $self->mirror,
+        cpanfile => $self->cpanfile,
     );
     $builder->update($self->install_path, @modules);
 
-    Carton::Lock->build_from_local($self->install_path, $prereqs)->write($self->lock_file);
+    Carton::Lock->build_from_local($self->install_path, $prereqs)->write($self->lockfile);
 }
 
 sub cmd_exec {
     my($self, @args) = @_;
 
-    my $lock = $self->find_lock
-        or $self->error("Can't find carton.lock: Run `carton install` to build the lock file.\n");
+    my $lock = $self->lockfile->load;
 
     # allows -Ilib
     @args = map { /^(-[I])(.+)/ ? ($1,$2) : $_ } @args;
@@ -383,38 +381,6 @@ sub cmd_exec {
     $UseSystem ? system(@args) : exec(@args);
 }
 
-sub find_cpanfile {
-    my $self = shift;
-
-    if (-e 'cpanfile') {
-        return 'cpanfile';
-    } else {
-        $self->error("Can't locate cpanfile\n");
-    }
-}
-
-sub find_lock {
-    my $self = shift;
-
-    if (-e $self->lock_file) {
-        my $lock;
-        try {
-            $lock = Carton::Lock->from_file($self->lock_file);
-        } catch {
-            $self->error("Can't parse carton.lock: $_\n");
-        };
-
-        return $lock;
-    }
-
-    return;
-}
-
-sub lock_file {
-    my $self = shift;
-    return 'carton.lock';
-}
-
 sub index_file {
     my $self = shift;
     $self->work_file("cache/modules/02packages.details.txt");
